﻿using CLU.Helpers;

using TreeSharp;

namespace CLU.Classes.Shaman
{
    using System.Linq;

    using CommonBehaviors.Actions;

    using global::CLU.GUI;

    using global::CLU.Lists;

    using Styx;
    using Styx.WoWInternals.WoWObjects;

    class Enhancement : RotationBase
    {
        private const int ItemSetId = 1071;

        public override string Name
        {
            get { return "Enhancement Shaman"; }
        }

        public override string KeySpell
        {
            get { return "Stormstrike"; }
        }

        public override float CombatMaxDistance
        {
            get { return 3.2f; }
        }

        // adding some help about cooldown management
        public override string Help
        {
            get
            {
                return "\n" +
                       "----------------------------------------------------------------------\n" +
                       "This Rotation will:\n" +
                       "2pc Tier set Bonus?: " + Items.Has2PcTeirBonus(ItemSetId) + "\n" +
                       "4pc Tier set Bonus?: " + Items.Has4PcTeirBonus(ItemSetId) + "\n" +
                       "1. Enchant Weapons: Windfury Weapon(MainHand) & Flametongue Weapon(OffHand)\n" +
                       "2. Totems: Strength of earth, Windfury, Mana Spring, Searing (with range check) \n" +
                       "3. AutomaticCooldowns has: \n" +
                       "==> UseTrinkets \n" +
                       "==> UseRacials \n" +
                       "==> UseEngineerGloves \n" +
                       "==> Feral Spirit \n" +
                       "4. AoE with Magma Totem, Chain Lightning, Fire Nova\n" +
                       "4. Heal using: Lightning Shield\n" +
                       "6. Best Suited for end game raiding\n" +
                       "NOTE: PvP uses single target rotation - It's not designed for PvP use. \n" +
                       "Credits to fluffyhusky, sjussju , Stormchasing\n" +
                       "----------------------------------------------------------------------\n";
            }
        }

        public override Composite SingleRotation
        {
            get
            {
               return new PrioritySelector(
                    // Pause Rotation
                    new Decorator(ret => SettingsFile.Instance.AutoManagePauseRotation, new ActionAlwaysSucceed()),

                    // For DS Encounters.
                    EncounterSpecific.ExtraActionButton(SettingsFile.Instance.AutoExtraActionButton),

                        new Decorator(
                            ret => Me.CurrentTarget != null && Units.IsTargetWorthy(Me.CurrentTarget),
                                new PrioritySelector(
                                        Item.UseTrinkets(),
                                        Spells.UseRacials(),
                                        Buff.CastBuff("Lifeblood", ret => true, "Lifeblood"), // Thanks Kink
                                        Item.UseEngineerGloves())),
                        // Interupts
                        Spells.CastInterupt("Wind Shear",                   ret => true, "Wind Shear"),
                        // Threat
                        Buff.CastBuff("Wind Shear",                         ret => Me.CurrentTarget != null && Me.CurrentTarget.ThreatInfo.RawPercent > 90, "Wind Shear (Threat)"),
                        // Weapon enchants
                        Buff.CastBuff("Windfury Weapon",                    ret => !Item.HasWeaponImbue(WoWInventorySlot.MainHand, "Windfury") && Item.HasSuitableWeapon(WoWInventorySlot.MainHand), "Windfury Weapon"),
                        Buff.CastBuff("Flametongue Weapon",                 ret => !Item.HasWeaponImbue(WoWInventorySlot.OffHand, "Flametongue") && Item.HasSuitableWeapon(WoWInventorySlot.OffHand), "Flametongue Weapon"),
                        // totems
                        Spells.CastTotem("Call of the Elements",            ret => Me.Totems.All(t => t.WoWTotem != WoWTotem.StrengthOfEarth && t.WoWTotem != WoWTotem.EarthElemental), "Call of the Elements"),
                        // AoE
                        new Decorator(
                                ret => !Me.IsMoving && Me.CurrentTarget != null && Units.CountEnnemiesInRange(Me.CurrentTarget.Location, 8) >= 2 && !BossList.IgnoreAoE.Contains(Unit.CurrentTargetEntry),
                                    new PrioritySelector(
                                        Spells.CastTotem("Magma Totem",     ret => Me.CurrentTarget != null && Units.CountEnnemiesInRange(Me.CurrentTarget.Location, 8) >= 6 && Me.Totems.All(t => t.WoWTotem != WoWTotem.Magma), "Magma Totem"),
                                        Spells.CastSpell("Chain Lightning", ret => Buff.PlayerCountBuff("Maelstrom Weapon") == 5, "Chain Lightning"),
                                        Spells.CastSpell("Flame Shock",     ret => true, "Flame Shock"),
                                        Spells.CastSpell("Lava Lash",       ret => Buff.TargetHasDebuff("Flame Shock"), "Lava Lash"),
                                        Spells.CastSpell("Fire Nova",       ret => true, "Fire Nova"),
                                        Spells.CastSpell("Stormstrike",     ret => true, "Stormstrike")
                                        )),
                        // Default Rotaion
			// Fire Elemental removed from List, if called manually it will still not be replaced, thanks to luv4tigger for the hint
                        //Spells.CastTotem("Fire Elemental Totem",            ret => Me.CurrentTarget != null && Units.IsTargetWorthy(Me.CurrentTarget) && Me.Totems.All(t => t.WoWTotem != WoWTotem.FireElemental), "Fire Elemental Totem"),
                        Spells.CastTotem("Earth Elemental Totem",           ret => Me.CurrentTarget != null && Units.IsTargetWorthy(Me.CurrentTarget) && Me.Totems.All(t => t.WoWTotem != WoWTotem.EarthElemental), "Earth Elemental Totem"),
                        Spells.CastTotem("Searing Totem",                   ret => Me.CurrentTarget != null && Me.CurrentTarget.Distance < Totems.GetTotemRange(WoWTotem.Searing) - 2f && !Me.Totems.Any(t => t.Unit != null && t.WoWTotem == WoWTotem.Searing && t.Unit.Location.Distance(Me.CurrentTarget.Location) < Totems.GetTotemRange(WoWTotem.Searing)) && Me.Totems.All(t => t.WoWTotem != WoWTotem.FireElemental), "Searing Totem"),
                        Spells.CastSpell("Stormstrike",                     ret => true, "Stormstrike"),
                        Spells.CastSpell("Lava Lash",                       ret => true, "Lava Lash"),
                        Spells.CastSpell("Lightning Bolt",                  ret => Buff.PlayerCountBuff("Maelstrom Weapon") == 5 || (Items.Has4PcTeirBonus(ItemSetId) ? Buff.PlayerCountBuff("Maelstrom Weapon") == 5 : Buff.PlayerCountBuff("Maelstrom Weapon") >= 4 && (Spell.SpellCooldown("Feral Spirit").TotalSeconds > 90 && Spell.SpellOnCooldown("Feral Spirit"))), "Lightning Bolt"),
                        Spells.CastSpell("Unleash Elements",                ret => true, "Unleash Elements"),
                        Spells.CastSpell("Flame Shock",                     ret => Buff.PlayerHasBuff("Unleash Flame"), "Flame Shock (Unleash Flame)"),
                        Buff.CastDebuff("Flame Shock",                      ret => true, "Flame Shock"),
                        Spells.CastSpell("Earth Shock",                     ret => Buff.TargetDebuffTimeLeft("Flame Shock").TotalSeconds > 5, "Earth Shock"),
                        Spells.CastSpell("Feral Spirit",                    ret => Me.CurrentTarget != null && Units.IsTargetWorthy(Me.CurrentTarget) && Unit.TimeToDeath(Me.CurrentTarget) > 30, "Feral Spirit"),
                        Spells.CastTotem("Earth Elemental Totem",           ret => Me.CurrentTarget != null && Units.IsTargetWorthy(Me.CurrentTarget) && Me.Totems.All(t => t.WoWTotem != WoWTotem.EarthElemental), "Earth Elemental Totem"),
                        Buff.CastBuff("Spiritwalker's Grace",               ret => Me.IsMoving, "Spiritwalker's Grace"),
                        Spells.CastSpell("Lightning Bolt",                  ret => Buff.PlayerCountBuff("Maelstrom Weapon") > 2, "Lightning Bolt")
                        );
            }
        }

        public override Composite Medic
        {
            get
            {
                return new Decorator(
                    ret => Me.HealthPercent < 100 && SettingsFile.Instance.AutoManageHealing,
                    new PrioritySelector(
                        new Decorator(
                            ret => Totems.NeedToRecallTotems,
                            new Sequence(
                                    new Action(ret => CLU.Instance.Log(" [Totems] Recalling Totems")),
                                    new Action(ret => Totems.RecallTotems()))),
                        Items.UseBagItem("Healthstone",         ret => Me.HealthPercent < 30, "Healthstone"),
                        Buff.CastBuff("Lightning Shield",       ret => true, "Lightning Shield"),
                        Buff.CastBuff("Shamanistic Rage",       ret => Me.CurrentTarget != null && (Me.HealthPercent < 60 || (Me.ManaPercent < 65 && Me.CurrentTarget.HealthPercent >= 75)), "Shamanistic Rage"),
                        Spells.CastSelfSpell("Healing Surge",   ret => Me.HealthPercent < 35, "Healing Surge"),
                        Buff.CastBuff("Windfury Weapon",        ret => !Item.HasWeaponImbue(WoWInventorySlot.MainHand, "Windfury") && Item.HasSuitableWeapon(WoWInventorySlot.MainHand), "Windfury Weapon"),
                        Buff.CastBuff("Flametongue Weapon",     ret => !Item.HasWeaponImbue(WoWInventorySlot.OffHand, "Flametongue") && Item.HasSuitableWeapon(WoWInventorySlot.OffHand), "Flametongue Weapon")
                        ));
            }
        }

        public override Composite PreCombat
        {
            get
            {
                return new Decorator(
                            ret => !Me.Mounted && !Me.Dead && !Me.Combat && !Me.IsFlying && !Me.IsOnTransport && !Me.HasAura("Food") && !Me.HasAura("Drink"),
                            new PrioritySelector(
                                new Decorator(
                                        ret => Totems.NeedToRecallTotems,
                                            new Sequence(
                                                new Action(ret => CLU.Instance.Log(" [Totems] Recalling Totems")),
                                                 new Action(ret => Totems.RecallTotems()))),
                                Buff.CastBuff("Lightning Shield",   ret => true, "Lightning Shield"),
                                Buff.CastBuff("Windfury Weapon",    ret => !Item.HasWeaponImbue(WoWInventorySlot.MainHand, "Windfury") && Item.HasSuitableWeapon(WoWInventorySlot.MainHand), "Windfury Weapon"),
                                Buff.CastBuff("Flametongue Weapon", ret => !Item.HasWeaponImbue(WoWInventorySlot.OffHand, "Flametongue") && Item.HasSuitableWeapon(WoWInventorySlot.OffHand), "Flametongue Weapon")
                                ));
            }
        }

        public override Composite Resting
        {
            get { return this.PreCombat; }
        }

        public override Composite PVPRotation
        {
            get { return this.SingleRotation; }
        }

        public override Composite PVERotation
        {
            get { return this.SingleRotation; }
        }
    }
}
